@@ -17,6 +17,7 @@
 *  2-Apr-91 | JDW : further changes
 *  4-Mar-91 | GWL : DOS allows odd inst addrs
 * 10-Oct-94 | nix : posicionador tridimensionale interface
+* 28-Apr-03 |  DM : true->TRUE, false->FALSE
 *****************************************************************************/
 
 #include "switches.h"
@@ -72,12 +73,12 @@ extern char *app_syntax;
 *  EXPORTS:
 *       
 *       cause(delay, routine, p1, p2, ..., p8)
-*       moxcdone -- set to true to quit
+*       moxcdone -- set to TRUE to quit
 *       eventtime -- ideallized current time
 *
 *****************************************************************************/
 
-#define SAFEMOXC true
+#define SAFEMOXC TRUE
 #define BREAKKEY 0x03
 
 int moxcdone;   /* flag to halt execution */
@@ -85,10 +86,10 @@ time_type eventtime;    /* time of current call -- used to avoid        */
                         /* timing errors due to finite execution speed  */
 time_type virttime;     /* virtual time of current call */
 timebase_type timebase; /* time base of current call */
-int mididecode = true;  /* whether to decode messages or just call midievent */
+int mididecode = TRUE;  /* whether to decode messages or just call midievent */
 
-int debug = false;
-int moxcdebug = false;
+int debug = FALSE;
+int moxcdebug = FALSE;
 time_type next_wakeup;
 timebase_type default_base;
 
@@ -214,10 +215,16 @@ void cause(delay_type delay, ...)
         gprintf(ERROR,"cause called with NULL routine\n");
         EXIT(1);
 #ifndef DOS     /* IBM allows odd addresses */
+#if (__APPLE__ != 1 || __i386__ != 1) /* Intel Mac allows odd addresses */
     } else if (((long) call->u.e.routine) & 1) {
         gprintf(ERROR, "cause called with bad routine address: 0x%lx\n",
                 call->u.e.routine);
+#ifndef GCC_MODEL_CPU
+#define GCC_MODEL_CPU "GCC_MODEL_CPU is undefined for this compilation"
+#endif
+		gprintf(ERROR, GCC_MODEL_CPU);
         EXIT(1);
+#endif
 #endif
     }
 #endif
@@ -280,10 +287,12 @@ void causepri(delay_type delay, int pri, ...)
         gprintf(ERROR,"cause called with NULL routine\n");
         EXIT(1);
 #ifndef DOS     /* IBM allows odd addresses */
+#if (__APPLE__ != 1 || __i386__ != 1) /* Intel Mac allows odd addresses */
     } else if (((long) call->u.e.routine) & 1) {
         gprintf(ERROR, "causepri called with bad routine address: 0x%lx\n",
                 call->u.e.routine);
         EXIT(1);
+#endif
 #endif
     }
 #endif
@@ -406,7 +415,7 @@ boolean moxcinit(argc, argv)
 #else
         ggetchar();
 #endif
-        return false;
+        return FALSE;
     }
     debug = cl_switch("debug");
     moxcdebug = cl_switch("moxc");
@@ -420,7 +429,7 @@ boolean moxcinit(argc, argv)
     ptInit();
 #endif
     moxcdone = 0;
-    return true;
+    return TRUE;
 }
 
 
@@ -478,7 +487,7 @@ private void decode()
 * poll for and decode midi keyboard input 
 ***********************************************/
 
-    while (getbuf(false, midi_data)) {
+    while (getbuf(FALSE, midi_data)) {
         /* only divide if necessary, divides take 100us on 8MHz 68000: */
         if (virttime == 0)
             virttime = real_to_virt(default_base, now);
@@ -635,16 +644,16 @@ private void decode()
 
 void quit()
 {
-    moxcdone = true;
+    moxcdone = TRUE;
 }
 
 /* moxcrun -- schedule events until done */
 /**/
 void moxcrun()
 {
-    moxcdone = false;
+    moxcdone = FALSE;
     while (!moxcdone && !abort_flag) {          /* test for finish */
-        if (!timebase_queue) moxcdone = true;
+        if (!timebase_queue) moxcdone = TRUE;
         else moxcwait(MAXTIME);         /* do work */
     }
 }
