@@ -5,9 +5,13 @@
 *-----------+---------------------------------------------------------------
 * 11-Mar-94 | Created Change Log
 * 11-Mar-94 | PLu : Added private to function defs.
+* 28-Apr-03 |  DM : Change #include's for portability
 ****************************************************************************/
 
-#include "stdio.h"
+#include "switches.h"
+
+#include <stdio.h>
+
 #include "cext.h"
 #include "userio.h"
 #include "midicode.h"
@@ -36,7 +40,7 @@ extern seq_type sequence; /* this is a global to be accessed by
                * routines called from the sequence */
 /* clock state: */
 extern time_type clock_ticksize;  /* millisec per tick shifted 16 bits */
-extern boolean clock_running;     /* true if clock is running */
+extern boolean clock_running;     /* TRUE if clock is running */
 extern boolean use_midi_clock;
 
 private void smfw_bend();
@@ -173,8 +177,8 @@ private void smfw_dotrack(seq)
     set_virttime(seq->timebase, 0L);
     seq->current = seq_events(seq);
     seq->noteoff_count = 0L;
-    seq->runflag = true;
-    seq->paused = true;
+    seq->runflag = TRUE;
+    seq->paused = TRUE;
     last_clock_event = 0L;
     last_event = 0L;
     if(debug)   gprintf(TRANS, "dotrack (reset) %d %ld (%lu) \n", 
@@ -283,7 +287,7 @@ private void smfw_process_event(seq)
         (((vc_voice(event->nvoice) - 16) == smfw_seq.track) 
             && (smfw_seq.track > 0)) ||
         /* acknowledge clock change on all tracks*/
-        (event->value == CLOCK_VALUE)) {
+        (event->value == CLOCK_VALUE && vc_ctrl(event->nvoice) == ESC_CTRL)) {
 
         /* process all current (and earlier) events */
         if (is_note(event)) {   /*** play a note or rest ***/
@@ -342,81 +346,81 @@ private void smfw_process_event(seq)
             switch (event->value) {
               time_type this_event;
               case CALL_VALUE:       /*called routine will write to midifile in execution */
-            sequence = seq;
-            (*(event->u.call.routine))(event->u.call.args);
-            break;
+                sequence = seq;
+                (*(event->u.call.routine))(event->u.call.args);
+                break;
               case CLOCK_VALUE:
-            clock_ticksize = event->u.clock.ticksize;
-            if(debug)    gprintf(TRANS, "clockevent! ticksize: %lu (time:%ld)\n",
-                clock_ticksize, virttime);
+                clock_ticksize = event->u.clock.ticksize;
+                if(debug)    gprintf(TRANS, "clockevent! ticksize: %lu (time:%ld)\n",
+                    clock_ticksize, virttime);
     
-            if (virttime > 0) {  /* any clock before this is already recorded in the header */
-                if (smfw_seq.track == 0) {             /* record clock event on tempo track = 0 */
-                /* cause clock write in half a newtick, because it was written .5 tick early*/
-                cause((delay_type) (clock_ticksize >> 17), smfw_clock_event, 
-                      last_tick_size, clock_ticksize);
-                last_tick_size = clock_ticksize; /*set new ticksize*/
-                } else { /*not on tempo track*/
-                this_event = ((virttime - last_clock_event) *
-                        ((2500L << 16) / last_tick_size)) / 100;
-                if(debug)   gprintf(TRANS, "track != 0: Lastev: %ld Thisev: %ld NewLast: %ld\n", 
-                last_event, this_event, this_event - last_event);
-                last_event = 0L - (this_event - last_event);
-                    last_clock_event = virttime;
-                /*last_event is negative, so will be ADDED to next this_event*/
+                if (virttime > 0) {  /* any clock before this is already recorded in the header */
+                    if (smfw_seq.track == 0) {             /* record clock event on tempo track = 0 */
+                    /* cause clock write in half a newtick, because it was written .5 tick early*/
+                    cause((delay_type) (clock_ticksize >> 17), smfw_clock_event, 
+                          last_tick_size, clock_ticksize);
+                    last_tick_size = clock_ticksize; /*set new ticksize*/
+                    } else { /*not on tempo track*/
+                    this_event = ((virttime - last_clock_event) *
+                            ((2500L << 16) / last_tick_size)) / 100;
+                    if(debug)   gprintf(TRANS, "track != 0: Lastev: %ld Thisev: %ld NewLast: %ld\n", 
+                    last_event, this_event, this_event - last_event);
+                    last_event = 0L - (this_event - last_event);
+                        last_clock_event = virttime;
+                    /*last_event is negative, so will be ADDED to next this_event*/
     
-                last_tick_size = clock_ticksize;
-                }               
-            } else if (debug) gprintf(TRANS, "IGNORED\n");/* if virttime <= 0 */
-            break;
+                    last_tick_size = clock_ticksize;
+                    }               
+                } else if (debug) gprintf(TRANS, "IGNORED\n");/* if virttime <= 0 */
+                break;
               case MACCTRL_VALUE:
-            if (!enabled) break;
-            if (debug) gprintf(TRANS, "MACCTRL %d: %d (time:%ld)\n", 
-                    event->u.macctrl.ctrl_number, event->u.macctrl.value, virttime);
-            smfw_deltatime();
-            putc(MIDI_CTRL | (voice - 1), smfw_seq.outfile);
-            putc(0xFF & event->u.macctrl.ctrl_number, smfw_seq.outfile);
-            putc(0xFF & event->u.macctrl.value, smfw_seq.outfile);
-            break;
+                if (!enabled) break;
+                if (debug) gprintf(TRANS, "MACCTRL %d: %d (time:%ld)\n", 
+                        event->u.macctrl.ctrl_number, event->u.macctrl.value, virttime);
+                smfw_deltatime();
+                putc(MIDI_CTRL | (voice - 1), smfw_seq.outfile);
+                putc(0xFF & event->u.macctrl.ctrl_number, smfw_seq.outfile);
+                putc(0xFF & event->u.macctrl.value, smfw_seq.outfile);
+                break;
               case MACRO_VALUE:
-            if (!enabled) break;
-            if (debug) gprintf(TRANS, "MACRO sent to...\n");
-            smfw_send_macro(event->u.macro.definition,
-              voice, event->u.macro.parameter, -1, 0);
-            break;
+                if (!enabled) break;
+                if (debug) gprintf(TRANS, "MACRO sent to...\n");
+                smfw_send_macro(event->u.macro.definition,
+                  voice, event->u.macro.parameter, -1, 0);
+                break;
               case CTRLRAMP_VALUE:
               case DEFRAMP_VALUE: {
-            int from, to;
-            if (!enabled) break;
-            step = event->u.ramp.step;
-            if (event->value == CTRLRAMP_VALUE) {
-            if(debug)   gprintf(TRANS, "CTRLRAMP (time:%ld)...", virttime);
-                from = event->u.ramp.u.ctrl.from_value;
-                to = event->u.ramp.u.ctrl.to_value;
-            } else {
-                if (debug) gprintf(TRANS, "DEFRAMP (time:%ld)...", virttime);
-                from = event->u.ramp.u.def.parameter[
-                    event->u.ramp.u.def.parm_num];
-                to = event->u.ramp.u.def.to_value;
-            }
-            delta = to - from;
-            increment = delta;
-            if (delta < 0) delta = -delta;
-            /* RBD - Note: Step is always non-zero */
-            n = event->u.ramp.dur / step;
-            increment = (increment << 8) / n;
-            smfw_ramp_event(seq, event, from << 8, to << 8, 
-                   increment, step, n);
-            seq->noteoff_count++;
-            break;
+                int from, to;
+                if (!enabled) break;
+                step = event->u.ramp.step;
+                if (event->value == CTRLRAMP_VALUE) {
+                if(debug)   gprintf(TRANS, "CTRLRAMP (time:%ld)...", virttime);
+                    from = event->u.ramp.u.ctrl.from_value;
+                    to = event->u.ramp.u.ctrl.to_value;
+                } else {
+                    if (debug) gprintf(TRANS, "DEFRAMP (time:%ld)...", virttime);
+                    from = event->u.ramp.u.def.parameter[
+                        event->u.ramp.u.def.parm_num];
+                    to = event->u.ramp.u.def.to_value;
+                }
+                delta = to - from;
+                increment = delta;
+                if (delta < 0) delta = -delta;
+                /* RBD - Note: Step is always non-zero */
+                n = event->u.ramp.dur / step;
+                increment = (increment << 8) / n;
+                smfw_ramp_event(seq, event, from << 8, to << 8, 
+                       increment, step, n);
+                seq->noteoff_count++;
+                break;
               }
               case SETI_VALUE:
-            seti_counter++; /*will be printed after writing is completed*/
-            *(event->u.seti.int_to_set) = event->u.seti.value;
-            break;
+                seti_counter++; /*will be printed after writing is completed*/
+                *(event->u.seti.int_to_set) = event->u.seti.value;
+                break;
               default:
-            gprintf(TRANS, "unexpected ESC_CTRL value\n");
-            break;
+                gprintf(TRANS, "unexpected ESC_CTRL value\n");
+                break;
             }
             break;
           default:
@@ -475,21 +479,25 @@ private void smfw_send_macro(ptr, voice, parameter, parm_num, value)
   int value;
 {
     register unsigned char code, *loc;
-    while (code = *ptr++) {
-    loc = ptr + *ptr;
-    ptr++;
-    if (code <= nmacroparms) {
-        code--;
-        *loc = (code == parm_num ? value : parameter[code]) & 0x7f;
-    } else if (code == nmacroparms + 1) {
-        *loc = ((voice - 1) & 0xF) | *loc;
-    } else {
-        code -= (nmacroparms + 2);
-        *loc = ((code == parm_num ? value : parameter[code]) >> 7) & 0x7F;
+    while ((code = *ptr++)) {
+        loc = ptr + *ptr;
+        ptr++;
+        if (code <= nmacroparms) {
+            code--;
+            *loc = (code == parm_num ? value : parameter[code]) & 0x7f;
+        }
+        else if (code == nmacroparms + 1) {
+            *loc = ((voice - 1) & 0xF) | *loc;
+        }
+        else {
+            code -= (nmacroparms + 2);
+            *loc = ((code == parm_num ? value : parameter[code]) >> 7) & 0x7F;
+        }
     }
-    }
-    if (ptr[1] == MIDI_SYSEX) smfw_exclusive(*ptr, ptr + 1);
-    else smfw_msg_write(*ptr, ptr[1], ptr[2], ptr[3]);
+    if (ptr[1] == MIDI_SYSEX)
+        smfw_exclusive(*ptr, ptr + 1);
+    else
+        smfw_msg_write(*ptr, ptr[1], ptr[2], ptr[3]);
 }
 
 
@@ -510,7 +518,10 @@ void seq_write_smf(seq, outfile)
 {
     time_type put_tick_size;
     int i;
-    time_type starting_ticksize = 1638400L;/*default midifile tempo 100*/
+    /* ticksize is milliseconds << 16, and tickrate = 24*tempo
+       60000ms/min / (24 * tempo) = 2500/tempo = 25
+       25 << 16 = 1638400 */
+    time_type starting_ticksize = 1638400L; /*default midifile tempo 100*/
     int track_count = 0;
     long track_count_marker;
     register event_type event;
@@ -530,15 +541,16 @@ void seq_write_smf(seq, outfile)
 
     event = seq_events(smfw_seq.seq);
 
-    /*search for clock events up till start of score*/
-    while(event->ntime <= 0){
-    if(debug)   gprintf(TRANS, "event (time:%ld)\n", event->ntime); 
-    if(event->value == CLOCK_VALUE) {
-        if(debug)   gprintf(TRANS, "clock %lu at 0\n", event->u.clock.ticksize);
-        starting_ticksize = event->u.clock.ticksize;
-        break;
-        }
-    event = event->next;
+    /* search for clock events up till start of score */
+    /* careful: there may be no events at all */
+    while(event && event->ntime <= 0){
+        if(debug)   gprintf(TRANS, "event (time:%ld)\n", event->ntime); 
+        if(vc_ctrl(event->nvoice) == ESC_CTRL && event->value == CLOCK_VALUE) {
+            if(debug)   gprintf(TRANS, "clock %lu at 0\n", event->u.clock.ticksize);
+            starting_ticksize = event->u.clock.ticksize;
+            break;
+            }
+        event = event->next;
     }
 
     putc(0x4D, smfw_seq.outfile); /*header:  MThd*/
@@ -559,49 +571,61 @@ void seq_write_smf(seq, outfile)
     putc(0x02, smfw_seq.outfile);/*division resolution of 600*/
     putc(0x58, smfw_seq.outfile);
 
-    for(i = 0; i < 17; i++){/*for each track..*/
-        if(((seq_used_mask(smfw_seq.seq) >> (i - 1)) & 0x1) || (i == 0)){
-        if(debug) gprintf(TRANS, "write track %d \n", i);
-        track_count++;
-        clock_ticksize = starting_ticksize;
-        last_tick_size = starting_ticksize;
-        putc(0x4D, smfw_seq.outfile);/*track header: MTrk*/
-        putc(0x54, smfw_seq.outfile);
-        putc(0x72, smfw_seq.outfile);
-        putc(0x6B, smfw_seq.outfile);
+    for (i = 0; i < 17; i++) { /* for each track... */
+        if (((seq_used_mask(smfw_seq.seq) >> (i - 1)) & 0x1) || (i == 0)) {
+            if (debug) gprintf(TRANS, "write track %d \n", i);
+            track_count++;
+            clock_ticksize = starting_ticksize;
+            last_tick_size = starting_ticksize;
+            putc(0x4D, smfw_seq.outfile);/*track header: MTrk*/
+            putc(0x54, smfw_seq.outfile);
+            putc(0x72, smfw_seq.outfile);
+            putc(0x6B, smfw_seq.outfile);
        
-        chunk_size_marker = ftell(smfw_seq.outfile);/*size of chunk will be written later*/
-        putc(0x00, smfw_seq.outfile);                               /*will be filled by chunk_size_marker*/
-        putc(0x00, smfw_seq.outfile);
-        putc(0x00, smfw_seq.outfile);
-        putc(0x00, smfw_seq.outfile);
+            chunk_size_marker = ftell(smfw_seq.outfile); 
+            /* size of chunk will be written later */
+            /* will be filled by chunk_size_marker */
+            putc(0x00, smfw_seq.outfile);
+            putc(0x00, smfw_seq.outfile);
+            putc(0x00, smfw_seq.outfile);
+            putc(0x00, smfw_seq.outfile);
 
-    if(i == 0) { /*tempo and time signature track*/
-        putc(0x00, smfw_seq.outfile);/* default time sig stuff*/
-        putc(0xFF, smfw_seq.outfile);
-        putc(0x58, smfw_seq.outfile);
-        putc(0x04, smfw_seq.outfile);
-        putc(0x04, smfw_seq.outfile);
-        putc(0x02, smfw_seq.outfile);
-        putc(0x18, smfw_seq.outfile);
-        putc(0x08, smfw_seq.outfile);
-        putc(0x00, smfw_seq.outfile);
+            if (i == 0) { /* tempo and time signature track */
+                putc(0x00, smfw_seq.outfile);/* default time sig stuff*/
+                putc(0xFF, smfw_seq.outfile);
+                putc(0x58, smfw_seq.outfile);
+                putc(0x04, smfw_seq.outfile);
+                putc(0x04, smfw_seq.outfile);
+                putc(0x02, smfw_seq.outfile);
+                putc(0x18, smfw_seq.outfile);
+                putc(0x08, smfw_seq.outfile);
+                putc(0x00, smfw_seq.outfile);
             
-        putc(0xFF, smfw_seq.outfile);/*TEMPO: inserted here in case default is used*/
-        putc(0x51, smfw_seq.outfile);
-        putc(0x03, smfw_seq.outfile);
-        put_tick_size = scale(clock_ticksize, 375L, 1024L);
-        putc((int) ((put_tick_size >> 16) & 0xFF), smfw_seq.outfile);
-        putc((int) ((put_tick_size >> 8) & 0xFF), smfw_seq.outfile);
-        putc((int) (put_tick_size & 0xFF), smfw_seq.outfile);
+                /* TEMPO: inserted here in case default is used */
+                putc(0xFF, smfw_seq.outfile);
+                putc(0x51, smfw_seq.outfile);
+                putc(0x03, smfw_seq.outfile);
+                /* ticksize is in ms<<16, so to get milliseconds per tick, it's
+                   ticksize / 65536. To get beat durations, multiply by 24 
+                   to get ticksize * 24 / 65536. To get microseconds, 
+                   multiply by 1000: ticksize * 24000 / 65536. Divide both
+                   constants by 64 to get ticksize * 375 / 1024 = 
+                   microseconds per quarter note.
+                */
+                put_tick_size = scale(clock_ticksize, 375L, 1024L);
+                putc((int) ((put_tick_size >> 16) & 0xFF), smfw_seq.outfile);
+                putc((int) ((put_tick_size >> 8) & 0xFF), smfw_seq.outfile);
+                putc((int) (put_tick_size & 0xFF), smfw_seq.outfile);
+            }
+            smfw_seq.track = i;
+            smfw_dotrack(smfw_seq.seq);
         }
-        smfw_seq.track = i;
-        smfw_dotrack(smfw_seq.seq);
-      }
     }
-    if(seti_counter) gprintf(TRANS, "%d SETI events IGNORED!\n", seti_counter);
+    if (seti_counter) 
+        gprintf(TRANS, "%d SETI events IGNORED!\n", seti_counter);
     seq_stop(smfw_seq.seq);
-    fseek(smfw_seq.outfile, track_count_marker, 0);/*go back and insert number of tracks*/
+    /* go back and insert number of tracks */
+    fseek(smfw_seq.outfile, track_count_marker, 0);
     putc(0xFF & track_count, smfw_seq.outfile);
     fclose(smfw_seq.outfile);
 }
@@ -627,7 +651,7 @@ private void writevarlen(value)
     for(;;) {
     if(debug) gprintf(TRANS, " byte ");
     putc((int) (buffer & 0xFF), smfw_seq.outfile);
-    if (buffer & 0x80)	buffer >>= 8;
+    if (buffer & 0x80)  buffer >>= 8;
     else break;
     }
     if(debug) gprintf(TRANS, "written!\n");
