@@ -1,30 +1,26 @@
-/* sound.h -- new fugue sound data type */
+/* sound.h -- new nyquist sound data type */
+
+/* CHANGE LOG
+ * --------------------------------------------------------------------
+ * 28Apr03  dm  changes for portability: moved some defns out of here
+ */
 
 #include <math.h>
 #include "stdefs.h"
 
+/* used for *AUDIO-MARKERS* */
+extern long sound_frames;
+extern double sound_srate;
 
-
-#ifndef max
-#define max(a, b) ((a) > (b) ? (a) : (b))
-#define min(a, b) ((a) < (b) ? (a) : (b))
+#if OSC
+extern int nosc_enabled; /* enable polling for OSC messages */
 #endif
 
-#ifndef abs
-#define abs(a) ((a) > 0 ? (a) : -(a))
-#endif
-
-#ifdef WINGUI
-void nyquist_printf(char *format, ...);
-#else
-#ifdef MACINTOSH
-void nyquist_printf(char *format, ...);
-#else
+#if USE_PRINTF
 #define nyquist_printf printf
 #endif
-#endif
 
-#define PERMS		 0644		/* -rw-r--r-- */
+#define PERMS            0644           /* -rw-r--r-- */
 
 /* default stop sample count (for clipping) */
 #define MAX_STOP 0x7FFFFFFF
@@ -38,6 +34,7 @@ void nyquist_printf(char *format, ...);
 /* conversion from float to integer */
 #define SCALE_FACTOR_TO_BYTE 127
 #define SCALE_FACTOR_TO_SHORT 32767
+#define SCALE_FACTOR_TO_24BIT 0x7FFFFF
 #define SCALE_FACTOR_TO_LONG 2147483647
 
 /* Note that the values assigned here are not arbitrary, but represent
@@ -111,6 +108,12 @@ void nyquist_printf(char *format, ...);
 #define INTERP_nrrr 63
 #define INTERP_srrr 127
 
+#define INTERP_nnnnnn 0 
+#define INTERP_ssssss 1365
+
+#define INTERP_nnnnnnnn 0
+#define INTERP_ssssssss 21845
+
 
 #define INTERP_MASK 3
 #define INTERP_SHIFT 2
@@ -128,7 +131,7 @@ typedef double promoted_sample_type;
 /* used by sndwrite.c for output buffers.  This should be
  * eliminated:
  */
-#define MAX_SND_CHANNELS 8
+#define MAX_SND_CHANNELS 24
 
 #define max_table_len 100000
 /* Set to 4 for debugging block allocation stuff, 1012? for
@@ -139,8 +142,8 @@ typedef double promoted_sample_type;
 /* #define max_sample_block_len 4 */
 
 /* Defines needed for xlisp */
-#define getsound(x)	((sound_type) getinst(x))
-#define xlgasound()	(testarg(typearg(soundp)))
+#define getsound(x)     ((sound_type) getinst(x))
+#define xlgasound()     (testarg(typearg(soundp)))
 
 typedef short SFDataType, *SFDataPtr;
 
@@ -148,17 +151,17 @@ typedef sample_type  sample_block_values[max_sample_block_len],
                     *sample_block_values_type;
 
 typedef struct {
-    long		refcnt;		                 /* reference count */
-    sample_block_values	samples;
+    long                refcnt;                          /* reference count */
+    sample_block_values samples;
 } sample_block_node, *sample_block_type;
  
 
 typedef struct snd_susp_struct {
-    void		(*fetch)();
-    void		(*keep_fetch)();
-    void		(*free)();
-    void                (*mark)();  /* marks LVAL nodes for GC */
-    void                (*print_tree)();    /* debugging */
+    void                (*fetch)(struct snd_susp_struct *, struct snd_susp_struct *);
+    void                (*keep_fetch)(struct snd_susp_struct *);
+    void                (*free)(struct snd_susp_struct *);
+    void                (*mark)(struct snd_susp_struct *);  /* marks LVAL nodes for GC */
+    void                (*print_tree)(struct snd_susp_struct *, int);    /* debugging */
     char *              name;       /* string name for debugging */
     long                toss_cnt;   /* return this many zeros, then compute */
     long                current;    /* current sample number */
@@ -170,20 +173,23 @@ typedef struct snd_susp_struct {
 
 
 typedef struct snd_list_struct {
-    sample_block_type	block;	/* pointer to block of samples */
+    sample_block_type   block;  /* pointer to block of samples */
     union {
-        struct snd_list_struct	*next;
-        snd_susp_type		susp;
-    }			u;
-    short		refcnt;
-    short		block_len;
-    boolean		logically_stopped;
+        struct snd_list_struct  *next;
+        snd_susp_type           susp;
+    }                   u;
+    short               refcnt;
+    short               block_len;
+    boolean             logically_stopped;
 } snd_list_node, *snd_list_type;
 
+extern snd_list_type list_watch; //DBY
+
 
 typedef struct table_struct {
     long refcount;  /* reference count */
-    double length;    /* number of samples in table (double allows fractional length)*/
+    double length;  /* number of samples in table 
+                       (double allows fractional length)*/
     sample_type samples[1]; /* arbitrary length array of sample */
 } table_node, *table_type;
 
@@ -194,31 +200,33 @@ typedef struct table_struct {
 #define UNKNOWN (-10-max_sample_block_len)
 
 typedef struct sound_struct {
-    sample_block_type	(*get_next)(/* struct sound_struct * snd, long * cnt */);
-    time_type		time;	/* logical starting time */
-    time_type		t0;	/* quantized time of first sample */
+    sample_block_type   (*get_next)(struct sound_struct *snd, long *cnt);
+    time_type           time;   /* logical starting time */
+    time_type           t0;     /* quantized time of first sample */
     long                stop;  /* stop (clipping) sample no. */
     time_type           true_t0;    /* exact time of first sample */
-    rate_type		sr;	/* sample rate */
-    long		current;	/* current sample number,
+    rate_type           sr;     /* sample rate */
+    long                current;        /* current sample number,
                                          if negative, then the first 
                                          -current samples must be dropped
                                          in order to find the first sample */
-    long		logical_stop_cnt; /* log stop sample no, -1=unknwn */
-    snd_list_type	list;	/* sample block list, starting at curr. samp */
-    sample_type		scale;	/* scale factor for the result */
+    long                logical_stop_cnt; /* log stop sample no, -1=unknwn */
+    snd_list_type       list;   /* sample block list, starting at curr. samp */
+    sample_type         scale;  /* scale factor for the result */
     long                prepend_cnt;    /* how many zeros to prepend */
-    /* function to use as get_next after prepended zeros have been generated: */
-    sample_block_type	(*after_prepend)
-                        (/* struct sound_struct * snd, long * cnt */);
+    /* function to use as get_next after prepended zeros are generated: */
+    sample_block_type   (*after_prepend)
+                        (struct sound_struct * snd, long * cnt);
     table_type table;   /* pointer to table-ized version of this sound */
-    long *extra;	/* used for extra state information, first word of extra state should
-                     * be the length of the extra state (see sound_unref())
-                     */
+    long *extra;        /* used for extra state information, first word of extra
+			   state should be the length of the extra state 
+			   (see sound_unref())
+                         */
 } sound_node, *sound_type;
 
 /* convert number of samples to memory size: */
-#define table_size_in_bytes(n) (sizeof(table_node) + sizeof(sample_type) * ((n) - 1))
+#define table_size_in_bytes(n) \
+    (sizeof(table_node) + sizeof(sample_type) * ((n) - 1))
 
 extern sample_block_type zero_block;
 extern sample_block_type internal_zero_block;
@@ -227,17 +235,21 @@ extern snd_list_type zero_snd_list;
 
 extern sound_type printing_this_sound;  /* debugging global */
 
+extern double sound_latency; /* controls output latency */
+double snd_set_latency(double latency); 
+/* LISP: (SND-SET-LATENCY FLONUM) */
+
 double compute_phase(double phase, double key, long n, double srate,
                      double new_srate, double freq, double *incr_ptr);
 
-boolean soundp();
+boolean soundp(LVAL);
 /* LISP: (SOUNDP ANY) */
 
 void snd_list_ref(snd_list_type list);
 void sound_unref(sound_type snd);
 void snd_list_unref(snd_list_type list);
 
-LVAL cvsound();
+LVAL cvsound(sound_type);
 extern LVAL a_sound;
 
 sample_block_type SND_get_next(sound_type snd, long * cnt);
@@ -245,9 +257,9 @@ sample_block_type SND_get_first(sound_type snd, long * cnt);
 sample_block_type SND_get_zeros(sound_type snd, long * cnt);
 sample_block_type SND_flush(sound_type snd, long * cnt);
 
-double hz_to_step();	/* LISP: (HZ-TO-STEP ANYNUM) */
+double hz_to_step(double);    /* LISP: (HZ-TO-STEP ANYNUM) */
 int interp_style(sound_type s, rate_type sr);
-void set_logical_stop_time(); /* LISP: (SND-SET-LOGICAL-STOP SOUND ANYNUM) */
+void set_logical_stop_time(sound_type sound, time_type when); /* LISP: (SND-SET-LOGICAL-STOP SOUND ANYNUM) */
 
 #define xlog(x) log(x)
 /* LISP: double (LOG FLONUM) */
@@ -300,10 +312,12 @@ void sound_print(LVAL snd_expr, long n);
     /* LISP: (SND-PRINT ANY FIXNUM) */
 void sound_play(LVAL snd_expr);
     /* LISP: (SND-PLAY ANY) */
-void stats();
+void stats(void);
     /* LISP: (STATS) */
 void sound_print_tree(sound_type snd);
     /* LISP: (SND-PRINT-TREE SOUND) */
+    
+void mark_audio_time(void);
 
 void sound_print_tree_1(sound_type snd, int n);
 
@@ -324,7 +338,7 @@ sound_type sound_zero(time_type t0, rate_type sr);
 
 #define susp_print_tree(s, n) (*((s)->print_tree))(s, n)
 
-double step_to_hz();
+double step_to_hz(double);
     /* LISP: (STEP-TO-HZ ANYNUM) */
 
 /* macros for access to samples within a suspension */
@@ -390,18 +404,39 @@ double step_to_hz();
  * if new ones are fetched, then run termination test and logical stop
  * test on signal and record results.  In this case, termination and logical
  * stop happen at the MAXIMUM of termination and logical stop times, resp.
+ *
+ * Originally, this code assumed that logical stops occurred on block boundaries,
+ * but because of the SET-LOGICAL-STOP function, which just writes a stop time
+ * into the sound_struct, the logical stop can be anywhere. As soon as the 
+ * logical stop is known, we want to propagate the value from the sound being
+ * read into the sound being computed. The propagation should set the logical
+ * stop of the computed sound to the MAX of any current value and the new 
+ * value. When the bit fields indicate that all logical stop times have been
+ * encountered, then the sound being computed will make the logical stop happen
+ * on a block boundary and set the flag on the block of samples where the stop
+ * occurs.
  */
 #define susp_check_term_log_block_samples(sound, sample_block_ptr, sample_ptr, sample_cnt, bit, all) \
     if (susp->sample_cnt == 0) { \
         susp_get_block_samples(sound, sample_block_ptr, sample_ptr, sample_cnt); \
-        if (susp->sound->logical_stop_cnt ==\
+/*OLD   if (susp->sound->logical_stop_cnt ==                  \
             susp->sound->current - susp->sample_cnt) { \
+*/ \
+        if (susp->sound->logical_stop_cnt != UNKNOWN && \
+            !(susp->logical_stop_bits & bit)) { \
             susp->logical_stop_bits |= bit; \
+/*OLD \
             if (susp->logical_stop_bits == all) { \
                 susp->susp.log_stop_cnt = (long) \
                  ((((susp->sound->current - susp->sample_cnt) / \
                    susp->sound->sr + susp->sound->t0) - \
-                   susp->susp.t0) * susp->susp.sr + 0.5); } } \
+                   susp->susp.t0) * susp->susp.sr + 0.5); \
+                assert(susp->susp.log_stop_cnt >= 0); } } \
+*/ \
+            susp->susp.log_stop_cnt = max(susp->susp.log_stop_cnt, \
+                    (((susp->sound->logical_stop_cnt / \
+                       susp->sound->sr + susp->sound->t0) - \
+                      susp->susp.t0) * susp->susp.sr + 0.5)); } \
         if (susp->sample_ptr == zero_block->samples) { \
             susp->terminate_bits |= bit; \
             if (susp->terminate_bits == all) { \
@@ -422,7 +457,7 @@ double step_to_hz();
  */
 #define logical_stop_cnt_cvt(sound) \
     (sound->logical_stop_cnt == UNKNOWN ? UNKNOWN : \
-     round((sound->logical_stop_cnt / sound->sr) * susp->susp.sr))
+     ROUND((sound->logical_stop_cnt / sound->sr) * susp->susp.sr))
 
 
 /* logical_stop_test tests to see if sound has logically stopped; if so,
