@@ -4,6 +4,8 @@
         Permission is granted for unrestricted non-commercial use
 
 HISTORY
+28-Apr-03   Mazzoni
+                Added declarations for path.c (new file)
 30-Mar-88	Dale Amon CMU-CSD
                 Set it up for unix. Picked _TURBOC_  because defs
                 are reasonable.
@@ -11,16 +13,9 @@ HISTORY
 
 /* system specific definitions */
 
-/* for PMAX */
-/* note: I'm using the following test to
- * detect a PMAX -- there may be a better way.
- */
-#if defined(mips) && !defined(sgi)
-char *getenv(char *name); /* for PMAX */
-#else
-#include <stdlib.h> /* normal case */
-#endif
-
+#include <stdlib.h> /* needed for getenv(); note that this was a problem
+    for PMAX implementation, but I assume PMAX is obsolete now. 
+    - RBD 16apr04 */
 #include <stdio.h>
 #include <ctype.h>
 #include <setjmp.h>
@@ -132,13 +127,8 @@ extern long ptrtoabs();
 #define XL_BIG_ENDIAN
 #endif
 
-#ifdef ANDREW
-#define STDERR stdout
-#endif
-
 /* Mac Metrowerks CW 6 */
 #ifdef __MWERKS__
-#include <stdlib.h>
 #define LSC
 #undef PATHNAMES
 #undef FILETABLE
@@ -153,14 +143,29 @@ extern long ptrtoabs();
 #endif
 
 /* Linux on Pentium */
-#ifdef __linux__
-#ifdef __i386__
-#define LINUX_INTEL
+#if defined(__linux__) || defined(__GLIBC__)
+#include <endian.h>
+#if __BYTE_ORDER == __LITTLE_ENDIAN
+#define XL_LITTLE_ENDIAN
+#else
+#define XL_BIG_ENDIAN
 #endif
 #endif
 
-#ifdef LINUX_INTEL
+/* Apple CC */
+#ifdef __APPLE__
+#define NNODES 2000
+#define AFMT "%lx"
+#define OFFTYPE long
+#define NIL (void *)0
+#define SAVERESTORE
+#include <sys/types.h>
+/* #if __BYTE_ORDER == __LITTLE_ENDIAN */
+#if defined(__LITTLE_ENDIAN__)
 #define XL_LITTLE_ENDIAN
+#else
+#define XL_BIG_ENDIAN
+#endif
 #endif
 
 /* default important definitions */
@@ -232,7 +237,11 @@ extern long ptrtoabs();
 #include "xldmem.h"
 
 /* program limits */
-#define STRMAX		100		/* maximum length of a string constant */
+#define STRMAX		250		/* maximum length of a string constant */
+/* this was 100 -- I had a perfectly good full path to init.lsp using
+   a directory structure created by Apple's Xcode that was about 108
+   characters, so I picked a bigger value. -RBD */
+
 #define HSIZE		1499		/* symbol hash table size */
 #define SAMPLE		1000		/* control character sample rate */
 
@@ -251,6 +260,7 @@ extern long ptrtoabs();
 #define FT_OBISNEW	13
 #define FT_OBCLASS	14
 #define FT_OBSHOW	15
+#define FT_OBISA        16
         
 /* macro to push a value onto the argument stack */
 #define pusharg(x)	{if (xlsp >= xlargstktop) xlargstkoverflow();\
@@ -456,7 +466,7 @@ LVAL xrdnohang(void);
 
 /* for extern.c */
 
-void inval_caches();
+void inval_caches(void);
 
 
 /* for xlbfun.c */
@@ -526,7 +536,7 @@ LVAL xwhen(void);
 LVAL xunless(void);
 LVAL xcase(void);
 LVAL xand(void);
-LVAL xor(void);
+LVAL x_or(void); // xor causes problems for gcc, so I renamed it. -RBD
 LVAL xif(void);
 LVAL xlet(void);
 LVAL xletstar(void);
@@ -572,7 +582,7 @@ void close_loadingfiles(void);
 /* xldmem.c */
 
 LVAL cons(LVAL x, LVAL y);
-LVAL cvstring(char *str);
+LVAL cvstring(const char *str);
 LVAL new_string(int size);
 LVAL cvsymbol(char *pname);
 LVAL cvsubr(LVAL (*fcn)(void), int type, int offset);
@@ -642,6 +652,7 @@ LVAL xmkstroutput(void);
 LVAL xgetstroutput(void);
 LVAL xgetlstoutput(void);
 LVAL xformat(void);
+LVAL xlistdir(void);
 
 
 /* xlimage.c */
@@ -662,9 +673,11 @@ int xlgetc(LVAL fptr);
 void xlungetc(LVAL fptr, int ch);
 int xlpeek(LVAL fptr);
 void xlputc(LVAL fptr, int ch);
+void xloutflush(LVAL fptr);
 void xlflush(void);
 void stdprint(LVAL expr);
 void stdputstr(char *str);
+void stdflush(void);
 void errprint(LVAL expr);
 void errputstr(char *str);
 void dbgprint(LVAL expr);
@@ -674,6 +687,8 @@ void trcputstr(char *str);
 
 
 /* xlisp.c */
+long xlrand(long range);
+double xlrealrand(void);
 void xlrdsave(LVAL expr);
 void xlevsave(LVAL expr);
 void xlfatal(char *msg);
@@ -815,12 +830,10 @@ int xlobsetvalue(LVAL pair, LVAL sym, LVAL val);
 LVAL obisnew(void);
 LVAL obclass(void);
 LVAL obshow(void);
+LVAL obisa(void);
 LVAL clnew(void);
 LVAL clisnew(void);
 LVAL clanswer(void);
-LOCAL LVAL entermsg(LVAL cls, LVAL msg);
-LOCAL LVAL sendmsg(LVAL obj, LVAL cls, LVAL sym);
-LOCAL LVAL evmethod(LVAL obj, LVAL msgcls, LVAL method);
 void obsymbols(void);
 void xloinit(void);
 
@@ -852,7 +865,7 @@ LVAL rmlpar(void);
 LVAL rmrpar(void);
 LVAL rmsemi(void);
 LVAL tentry(int ch);
-int isnumber(char *str, LVAL *pval);
+int xlisnumber(char *str, LVAL *pval);
 void defmacro(int ch, LVAL type, int offset);
 LVAL callmacro(LVAL fptr, int ch);
 void xlrinit(void);
@@ -948,6 +961,7 @@ LVAL findprop(LVAL sym, LVAL prp);
 
 /* xlsys.c */
 
+LVAL xget_env(void);
 LVAL xload(void);
 LVAL xtranscript(void);
 LVAL xtype(void);
@@ -959,6 +973,9 @@ LVAL xaddrs(void);
 
 /* macstuff, unixstuff, winstuff */
 
+extern const char os_pathchar;
+extern const char os_sepchar;
+
 void osinit(char *banner);
 void oserror(char *msg);
 void osfinish(void);
@@ -966,22 +983,27 @@ int osclose(FILE *fp);
 void osflush(void);
 void oscheck(void);
 int osaputc(int ch, FILE *fp);
+void osoutflush(FILE *fp);
 int osbputc(int ch, FILE *fp);
 void ostputc(int ch);
+void ostoutflush(void);
 int osagetc(FILE *fp);
 int osbgetc(FILE *fp);
 int ostgetc(void);
 void ossymbols(void);
-long osrand(long rseed);
-/* Added by Ning Hu		May.2001*/
 LVAL xlinfo(void);
 LVAL xsetdir(void);
-/* Added End */
-#ifdef WINDOWS
-void get_xlisp_path(char *p, long p_max);
-#endif
-
+int osdir_list_start(char *path);
+char *osdir_list_next(void);
+void osdir_list_finish(void);
+LVAL xosc_enable(void);
+LVAL xget_temp_path(void);
+LVAL xfind_in_xlisp_path(void);
 
+/* These are now implemented in path.c   -dmazzoni */
+const char *return_xlisp_path(void);
+const char *find_in_xlisp_path(const char *fname);
+void set_xlisp_path(const char *p);
 
 /* local.c - these procedures are specific to each implementation */
 
